use strict;
use warnings;
use Test::More tests => 38;

# $Id$

$ENV{EPOLL_NO_COMMIT} = 1;

use_ok 'Epoll', 'Epoll';
use_ok 'Epoll::DB';
use_ok 'Catalyst::Test', 'Epoll'; 

my $vote = Epoll->model('Vote');

ok(my $pollid = $vote->_create_poll('test@', 'Test poll', 'password'),
    "Can get new poll id");

ok(my $poll = $vote->poll($pollid), "Can get poll object");

ok(my $vote_uid = $poll->uid, "Can get uid");

ok($poll->addupd_voting('user@', 'id'), "Can add a voting");
ok(my $votingpass = $poll->voting_from_mail('user@')->gen_passwd,
    'can get voting password');

diag("Test request before poll");
ok( request('/')->is_success, 'Request should succeed' );
ok( request("/poll/$pollid")->is_success, "Request /poll/$pollid should succeed" );
ok( request("/poll/$vote_uid")->is_success, "Request /poll/$vote_uid should succeed" );

ok( request("/admin/$pollid?vpass$pollid=password")->is_success,
    "Request /admin/$pollid should succeed" );
ok( request("/admin/$pollid/voting?vpass$pollid=password")->is_success,
    "Request /admin/$pollid/voting should succeed" );
ok( request("/admin/$pollid/ballot?vpass$pollid=password")->is_success,
    "Request /admin/$pollid/ballot should succeed" );

ok( request("/ballot/$pollid")->is_success, "Request /admin/$pollid should succeed" );
ok( request("/ballot/$pollid?mail=user@;password=$votingpass")->is_success,
    "Request /admin/$pollid should succeed" );

# Be sure today is included:
diag("Test request during poll");
ok($poll->param( start => '1/1/2000', end => '2/1/3000'),
    "Can set poll date");

ok( request('/')->is_success, 'Request should succeed' );
ok( request("/poll/$pollid")->is_success, "Request /poll/$pollid should succeed" );
ok( request("/poll/$pollid/report.pdf")->is_success, "Request /poll/$pollid/report.pdf should succeed" );

ok( request("/admin/$pollid?vpass$pollid=password")->is_success,
    "Request /admin/$pollid should succeed" );
ok( request("/admin/$pollid/voting?vpass$pollid=password")->is_success,
    "Request /admin/$pollid/voting should succeed" );
ok( request("/admin/$pollid/ballot?vpass$pollid=password")->is_success,
    "Request /admin/$pollid/ballot should succeed" );

ok( request("/ballot/$pollid")->is_success, "Request /admin/$pollid should succeed" );
ok( request("/ballot/$vote_uid")->is_success, "Request /admin/$vote_uid should succeed" );
ok( request("/ballot/$pollid?mail=user@;password=$votingpass")->is_success,
    "Request /admin/$pollid should succeed" );


diag("Test request after poll");
ok($poll->param( start => '1/1/1900', end => '2/1/1900'),
    "Can set poll date");

ok( request('/')->is_success, 'Request should succeed' );
ok( request("/poll/$pollid")->is_success, "Request /poll/$pollid should succeed" );
ok( request("/poll/$vote_uid")->is_success, "Request /poll/$vote_uid should succeed" );
ok( request("/poll/$pollid/report.pdf")->is_success, "Request /poll/$pollid/report.pdf should succeed" );
ok( request("/poll/$pollid/results.pdf")->is_success, "Request /poll/$pollid/results.pdf should succeed" );
ok( request("/poll/$vote_uid/results.pdf")->is_success, "Request /poll/$vote_uid/results.pdf should succeed" );

ok( request("/admin/$pollid?vpass$pollid=password")->is_success,
    "Request /admin/$pollid should succeed" );
ok( request("/admin/$pollid/voting?vpass$pollid=password")->is_success,
    "Request /admin/$pollid/voting should succeed" );
ok( request("/admin/$pollid/ballot?vpass$pollid=password")->is_success,
    "Request /admin/$pollid/ballot should succeed" );

ok( request("/ballot/$pollid")->is_success, "Request /admin/$pollid should succeed" );
ok( request("/ballot/$pollid?mail=user@;password=$votingpass")->is_success,
    "Request /admin/$pollid should succeed" );

